# -*- coding: utf-8 -*-
# Generated by Django 1.11.7 on 2018-02-07 18:21
from __future__ import unicode_literals

from django.contrib.auth.management import create_permissions
from django.db import migrations

new_archive_manager_perms = [
    "add_collection",
    "change_collection",
    "delete_collection",
]


def update_archive_manager_group(apps, schema_editor):
    Group = apps.get_model("auth", "Group")
    Permission = apps.get_model("auth", "Permission")

    # make sure permissions are created before loading the fixture
    # which references them
    # (when running migrations all at once, permissions may not yet exist)
    for app_config in apps.get_app_configs():
        app_config.models_module = True
        create_permissions(app_config, apps=apps, verbosity=0)
        app_config.models_module = None

    archive_mgr = Group.objects.get(name="Archive Manager")
    permissions = []
    for codename in new_archive_manager_perms:
        # using explicit get so that there will be an error if an
        # expected permission is not found
        permissions.append(Permission.objects.get(codename=codename))

    # add the new permissions without removing existing ones
    archive_mgr.permissions.add(*permissions)
    archive_mgr.save()


class Migration(migrations.Migration):

    dependencies = [
        ("common", "0002_create_script_user"),
    ]

    operations = [
        migrations.RunPython(
            update_archive_manager_group, reverse_code=migrations.RunPython.noop
        ),
    ]
